#include <stdio.h>
#include <stdlib.h>
#include "digraph.h"

#define HASH_MULT  314159     /* random multiplier */
#define HASH_PRIME 516595003  /* the 27182818th prime; it's less than $2^{29}$ */

static Vertex hash_head[maxV];
static Vertex hash_link[maxV];

/* http:.../~pf/algoritmos_para_grafos/aulas/digraphdatastructs.html */

/* Esta função devolve (o endereço de) um novo digrafo com 
   vértices 0,..,V-1 e nenhum arco. */

Digraph DIGRAPHinit (int V) { 
  Digraph G = malloc(sizeof *G);
  G->V = V; 
  G->A = 0;
  G->adj = MATRIXint(V, V, 0);
  return G;
}

/* A função abaixo aloca uma matriz com linhas 0..r-1 e colunas 0..c-1. 
   Cada elemento da matriz recebe valor val. */

int **MATRIXint (int r, int c, int val) { 
  Vertex i, j;
  int **m = malloc(r * sizeof(int *));
  for (i = 0; i < r; i++)
    m[i] = malloc(c * sizeof(int));
  for (i = 0; i < r; i++)
    for (j = 0; j < c; j++)
      m[i][j] = val;
  return m;
}

/* Esta função insere um arco v-w no digrafo G. 
   Se v == w ou o digrafo já tem arco v-w, a função não faz nada.  
   É claro que v e w não podem ser negativos e devem ser menores que G->V. */

void DIGRAPHinsertA (Digraph G, Vertex v, Vertex w) { 
  if (v != w && G->adj[v][w] == 0) {
    G->adj[v][w] = 1; 
    G->A++;
  }
}

/* Esta função remove do digrafo G o arco que tem ponta inicial 
   v e ponta final w.  Se não existe tal arco, a função nada faz. */

void DIGRAPHremoveA (Digraph G, Vertex v, Vertex w) { 
  if (G->adj[v][w] == 1) {
    G->A--;
    G->adj[v][w] = 0; 
  }
}

/* Para cada vértice v do digrafo G, esta função imprime,
   em uma linha, todos os vértices adjacentes a v. */

void DIGRAPHshow (Digraph G) { 
  Vertex v, w; 
  for (v = 0; v < G->V; v++) {
    printf("%2d:", v);
    for (w = 0; w < G->V; w++)
      if (G->adj[v][w] == 1) 
	printf(" %2d", w);
    printf("\n");
  }
}

void DIGRAPHfree (Digraph G)
{
    Vertex v;

    for (v = 0; v < G->V; v++) {
        free(G->adj[v]);
    }
    free(G->adj);
    free(G);
}

void DIGRAPHhash_setup (Digraph G)
{
    if (G && G->V > 0) 
    {
        register Vertex v;
        for (v = 0; v < G->V; v++) hash_head[v] = -1;
        for (v = 0; v < G->V; v++) hash_in(v, G);
    }
}

void DIGRAPHhash_in (Vertex v, Digraph G)
{
    register char *t=url[v];
    register Vertex u;
    register long h;

    /*Find vertex u, whose location is the hash code for string t;*/
    for (h = 0; *t; t++) {
        h += (h ^ (h >> 1)) + HASH_MULT * (unsigned char)*t;
        while (h >= HASH_PRIME) h -= HASH_PRIME;
    }
    u = h % G->V;

    hash_link[v] = hash_head[u];
    hash_head[u] = v;
}

Vertex DIGRAPHhash_out (char* s, Digraph G)
{
    register char*t= s;
    register Vertex  u;
    register long h;

    /*Find vertex u, whose location is the hash code for string t;*/
    for (h = 0; *t; t++) {
        h += (h ^ (h >> 1)) + HASH_MULT * (unsigned char)*t;
        while (h >= HASH_PRIME) h -= HASH_PRIME;
    }
    u = h % G->V;

    for (u = hash_head[u]; u != -1; u = hash_link[u]) {
        if (strcmp(s, G->URL[u]) == 0) return u;
    }

    return -1;
}

Vertex DIGRAPHhash_lookup (char* s, Digraph G)
{
    if (G && G->V > 0) {
        register Vertex v;
        v = hash_out(s, G);
        return v;
    }
    else return -1;
}

