/* http:.../~pf/algoritmos_para_grafos/aulas/digraphdatastructs.html */

#ifndef _digraph_h
#define _digraph_h

/* Vértices são representados por objetos do tipo Vertex.  
   Os vértices de um digrafo são 0,1,..,V-1. */

#define Vertex int

/* A estrutura digraph representa um digrafo. 
 * O campo adj é um ponteiro para a matriz de adjacência do digrafo. 
 * O campo V contém o número de vértices e o campo A contém o 
 * número de arcos do digrafo. 
 */

struct digraph {
    int V; 
    int A; 
    int** adj; 
};

/* Um objeto do tipo Digraph contém o endereço de um digraph. */

typedef struct digraph* Digraph;


/* Esta função devolve (o endereço de) um novo digrafo com 
 * vértices 0,..,V-1 e nenhum arco. 
 */

Digraph DIGRAPHinit (int V);

/* A função abaixo aloca uma matriz com linhas 0..r-1 e colunas 0..c-1. 
 * Cada elemento da matriz recebe valor val. 
 */

int** MATRIXint (int r, int c, int val);

/* Esta função insere um arco v-w no digrafo G. 
 * Se v == w ou o digrafo já tem arco v-w, a função não faz nada.  
 * É claro que v e w não podem ser negativos e devem ser menores que G->V. 
 */

void DIGRAPHinsertA (Digraph G, Vertex v, Vertex w);

/* Esta função remove do digrafo G o arco que tem ponta inicial 
 * v e ponta final w.  Se não existe tal arco, a função nada faz.
 */

void DIGRAPHremoveA (Digraph G, Vertex v, Vertex w);

/* Para cada vértice v do digrafo G, esta função imprime,
 * em uma linha, todos os vértices adjacentes a v. 
 */

void DIGRAPHshow (Digraph G);

/* Recebe um digrafo G e libera a memoria alocada 
 * a ele (matriz de adjacencia e estrutura
 */

void DIGRAPHfree (Digraph G);

void DIGRAPHhash_setup (Digraph G);

void DIGRAPHhash_in (Vertex v, Digraph G);

Vertex DIGRAPHhash_out (char* s, Digraph G);

Vertex hash_lookup (char* s, Digraph G);

#endif
