#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "digraph.h"

int 
outdegree (Digraph G, Vertex v);

float 
norm1(float* a, int n);

void 
print_rank(float* rank, int V);

void 
print_out(int* out, int V);

void 
pagerank(Digraph G, int *out, float* rank);

float*
point_subt(float* a, float* b, int size);

int main()
{
    Digraph G;
    int    V; /* numero de vertice */
    int    A;
    Vertex v;
    Vertex w;
    int *out;
    float* rank;
    float* old_rank;
    float* diff;
    int i;
    float epsilon;
    float rank_sum;
    int iter = 0;
    float norm1_diff;


    /* Read input */
    scanf("%d", &V);
    scanf("%d", &A);

    G = DIGRAPHinit(V);

    for (i = 0; i < A; i++) {
        scanf("%d %d", &v, &w);
        DIGRAPHinsertA(G,v,w);
    }

    rank = malloc(V * sizeof(float));
    old_rank = malloc(V * sizeof(float));
    out = malloc(V * sizeof(int));
    scanf("%f", &epsilon);

    /* Compute naive PageRank */

    /* Initialize*/
    for (v = 0; v < G->V; v++) {
        rank[v] = 1.0 / G->V;
        out[v]  = outdegree(G,v);
    }
    print_rank(rank, G->V);
    print_out(out, G->V);


    /* Iteration */
    norm1_diff = norm1(rank, G->V);

    while (norm1_diff > epsilon) {
        rank_sum = 0;
        for (v = 0; v < G->V; v++) {
            old_rank[v] = rank[v];
        }
        pagerank(G, out, rank);
        diff = point_subt(rank, old_rank, G->V);
        norm1_diff = norm1(diff, G->V);
        iter++;
        printf("Iterations = %d\n", iter);
        print_rank(rank, G->V);
        free(diff);
    }

    /* Free routines */
    free(rank);
    free(old_rank);
    free(out);
    DIGRAPHfree(G);
    return 0;
}


void pagerank(Digraph G, int *out, float* rank)
{
    Vertex v, w;
    float* new_rank;
    float c;

    new_rank = malloc(G->V * sizeof(float));
    c = 0;
    for (w = 0; w < G->V; w++) {
        new_rank[w] = 0;
        for (v = 0; v < G->V; v++) {
            if (G->adj[v][w] == 1) {
                new_rank[w] += rank[v]/out[v];
            }
        }
        c += new_rank[w];
    }

    printf("c = %.3f\n", c);
    for (v = 0; v < G->V; v++) {
        rank[v] = new_rank[v] / c;
    }
    free(new_rank);
}

int outdegree (Digraph G, Vertex v)
{
    Vertex w;
    int degree = 0;

    for (w = 0; w < G->V; w++) {
        degree += G->adj[v][w];
    }
    return degree;
}

float norm1 (float* a, int n)
{
    int i;
    float sum = 0;

    for (i = 0; i < n; i++) {
        sum += fabs(a[i]);
    }
    return sum;
}

void print_rank(float* rank, int V)
{
    Vertex v;

    for (v = 0; v < V; v++) {
        printf("R(%d) = %.3f\n", v, rank[v]);
    }
    printf("\n");
}



void print_out(int* out, int V)
{
    Vertex v;

    for (v = 0; v < V; v++) {
        printf("out[%d] = %d\n", v, out[v]);
    }
    printf("\n");
}

float* point_subt(float* a, float* b, int size)
{
    int i;
    float* diff = malloc(size * sizeof(float));

    for (i = 0; i < size; i++) {
        diff[i] = a[i] - b[i];
    }
    return diff;
}

