#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "digraph.h"

int 
outdegree (Digraph G, Vertex v);

float 
norm1(float* a, int n);

void 
print_rank(float* rank, int V);

void 
print_out(int* out, int V);

void 
pagerank(Digraph G, float* rank, float* E);

void
point_sub(float* a, float* b, float* c, int size);

void
point_sum(float* a, float* b, float* c, int size);

float*
matrix_mult(int** A, float* R, int n);

void
normalize(float* a, int n);

int main()
{
    Digraph G;
    int    V; /* numero de vertice */
    int    A;
    Vertex v;
    Vertex w;
    int *out;
    float* rank;
    float* old_rank;
    float* diff;
    float* E;
    int i;
    float epsilon;
    float d;
    int iter = 0;
    float norm1_diff;


    /* Read input */
    scanf("%d", &V);
    scanf("%d", &A);

    G = DIGRAPHinit(V);

    for (i = 0; i < A; i++) {
        scanf("%d %d", &v, &w);
        DIGRAPHinsertA(G,v,w);
    }

    rank = malloc(V * sizeof(float));
    old_rank = malloc(V * sizeof(float));
    out = malloc(V * sizeof(int));
    E = malloc(V * sizeof(float));
    scanf("%f", &epsilon);

    /* Compute not-so-naive PageRank */
    
    /* 
     * - Arrumar a matriz para A[u][v] = 1 / N[u] || 0 (nao é bem isso...)
     *   paper ta meio errado
     * - Arrumar o vetor E
     * - Checar a normalizacao
     * - entender certo o 'c'
     * - ler de novo o artigo na wiki
     * - Entender o vetor E
     * - PageRank Uncovered
     * - the anatomy of a large-scale hypertextual web search engine [BP]
     * - ESCREVER ALGUMA COISA
     * - email do coelho
     */

    /* Initialize */
    for (v = 0; v < G->V; v++) {
        rank[v] = 1.0 / G->V;
        out[v]  = outdegree(G, v);
        E[v] = 0.0;
    }
    print_rank(rank, G->V);
    print_out(out, G->V);


    /* Iteration */
    norm1_diff = norm1(rank, G->V);
    diff = malloc(G->V * sizeof(float));

    while (norm1_diff > epsilon) {
        for (v = 0; v < G->V; v++) {
            old_rank[v] = rank[v];
        }

        /* NORMALIZAR EM ALGUM CANTO AQUI!!! */
        pagerank(G, rank, E);
        d = norm1(old_rank, G->V) - norm1(rank, G->V);
        for (v = 0; v < G->V; v++) {
            rank[v] += (d * E[v]);
        }
        point_sub(rank, old_rank, diff, G->V);
        norm1_diff = norm1(diff, G->V);
        iter++;
        printf("Iterations = %d\n", iter);
        print_rank(rank, G->V);
    }

    /* Free routines */
    free(diff);
    free(rank);
    free(old_rank);
    free(out);
    free(E);
    DIGRAPHfree(G);
    return 0;
}


void pagerank(Digraph G, float* rank, float* E)
{
    Vertex v;
    float* new_rank;

    new_rank = matrix_mult(G->adj, rank, G->V);
    /* point_sum(new_rank, E, new_rank, G->V); 
    normalize(new_rank, G->V); */
    for (v = 0; v < G->V; v++) {
        rank[v] = new_rank[v];
    }
    free(new_rank);
}

int outdegree (Digraph G, Vertex v)
{
    Vertex w;
    int degree = 0;

    for (w = 0; w < G->V; w++) {
        degree += G->adj[v][w];
    }
    return degree;
}

float norm1 (float* a, int n)
{
    int i;
    float sum = 0;

    for (i = 0; i < n; i++) {
        sum += fabs(a[i]);
    }
    return sum;
}

void print_rank(float* rank, int V)
{
    Vertex v;

    for (v = 0; v < V; v++) {
        printf("R(%d) = %.3f\n", v, rank[v]);
    }
    printf("\n");
}

void print_out(int* out, int V)
{
    Vertex v;

    for (v = 0; v < V; v++) {
        printf("out[%d] = %d\n", v, out[v]);
    }
    printf("\n");
}

void point_sub(float* a, float* b, float* c,  int size)
{
    int i;

    for (i = 0; i < size; i++) {
        c[i] = a[i] - b[i];
    }
}

void point_sum(float* a, float* b, float* c, int size)
{
    int i;

    for (i = 0; i < size; i++) {
        c[i] = a[i] + b[i];
    }
}

float* matrix_mult (int** A, float* R, int n)
{
    int i, j;
    float partial_sum;
    float* result = malloc(n * sizeof(float));

    for (i = 0; i < n; i++) {
        partial_sum = 0;
        for (j = 0; j < n; j++) {
            partial_sum += A[i][j] * R[j];
        }
        result[i] = partial_sum;
    }
    return result;
}

void normalize (float* a, int n)
{
    int i;
    float c = 0;

    for (i = 0; i < n; i++) {
        c += a[i];
    }
    for (i = 0; i < n; i++) {
        a[i] /= c;
    }
}

